From 50f38667510057beea9c88bff8b7ee8cda825939 Mon Sep 17 00:00:00 2001
From: Alexander Korotkov <akorotkov@postgresql.org>
Date: Fri, 10 May 2024 03:08:15 +0300
Subject: [PATCH v2 3/4] Amcheck: Don't load rightpage into BtreeCheckState

For cross-page unique constraint check use a local variable in the
similar way as implemented in bt_right_page_check_scankey().

Reported-by: Peter Geoghegan
---
 contrib/amcheck/verify_nbtree.c | 14 +++++++++-----
 1 file changed, 9 insertions(+), 5 deletions(-)

diff --git a/contrib/amcheck/verify_nbtree.c b/contrib/amcheck/verify_nbtree.c
index b433cb33254..977f8b6799d 100644
--- a/contrib/amcheck/verify_nbtree.c
+++ b/contrib/amcheck/verify_nbtree.c
@@ -1910,6 +1910,8 @@ bt_target_page_check(BtreeCheckState *state)
 				/* The first key on the next page is the same */
 				if (_bt_compare(state->rel, rightkey, state->target, max) == 0 && !rightkey->anynullkeys)
 				{
+					Page		rightpage;
+
 					/*
 					 * Do the bt_entry_unique_check() call if it was
 					 * postponed.
@@ -1918,19 +1920,21 @@ bt_target_page_check(BtreeCheckState *state)
 						bt_entry_unique_check(state, itup, state->targetblock, offset, &lVis);
 
 					elog(DEBUG2, "cross page equal keys");
-					state->target = palloc_btree_page(state,
-													  rightblock_number);
-					topaque = BTPageGetOpaque(state->target);
+					rightpage = palloc_btree_page(state,
+												  rightblock_number);
+					topaque = BTPageGetOpaque(rightpage);
 
 					if (P_IGNORE(topaque) || !P_ISLEAF(topaque))
 						break;
 
 					itemid = PageGetItemIdCareful(state, rightblock_number,
-												  state->target,
+												  rightpage,
 												  rightfirstoffset);
-					itup = (IndexTuple) PageGetItem(state->target, itemid);
+					itup = (IndexTuple) PageGetItem(rightpage, itemid);
 
 					bt_entry_unique_check(state, itup, rightblock_number, rightfirstoffset, &lVis);
+
+					pfree(rightpage);
 				}
 			}
 		}
-- 
2.39.3 (Apple Git-145)

